import unittest
from os.path import getsize
from typing import List, Dict

from mg_file.file.csv_file import CsvFile
from mg_file.file.json_file import JsonFile
from mg_file.file.pickle_file import PickleFile
from mg_file.file.txt_file import TxtFile


class TestFile(unittest.TestCase):

    def __init__(self, methodName: str = ...) -> None:
        super().__init__(methodName)
        # Имя фалйа
        self.name_file = "test.txt"
        # Данные для теста
        self.test_str: str = "ninja cjj,output На двух языках 1#1^23 !23№эЭ123'"

    # Этот метод запускается ПЕРЕД каждой функции теста
    def setUp(self) -> None:
        self.testClassFile = TxtFile(self.name_file)
        self.testClassFile.deleteFile()
        self.testClassFile.createFileIfDoesntExist()

    def test_sizeFile(self):
        # Проверка определение размера файла
        self.testClassFile.writeFile(self.test_str)
        self.assertEqual(self.testClassFile.sizeFile(), getsize(self.testClassFile.name_file))

    def test_deleteFile_and_checkExistenceFile(self):
        # Проверка удаление файла
        self.assertEqual(self.testClassFile.checkExistenceFile(), True)
        self.testClassFile.deleteFile()
        self.assertEqual(self.testClassFile.checkExistenceFile(), False)

    def test_writeFile(self):
        # Проверка записи в файл
        self.testClassFile.writeFile(self.test_str)
        self.assertEqual(self.test_str, self.testClassFile.readFile())

    def test_appendFile(self):
        # Проверка дозaписи в файл
        test_str: str = self.test_str
        self.testClassFile.writeFile(test_str)
        self.testClassFile.appendFile(test_str)
        test_str += test_str
        self.assertEqual(test_str, self.testClassFile.readFile())

    def test_readBinaryFile_and_writeBinaryFile(self):
        # Проверка записи и чтения в двоичном режиме
        self.testClassFile.writeBinaryFile(self.test_str.encode())
        self.assertEqual(self.test_str.encode(), self.testClassFile.readBinaryFile())

    def test_appendBinaryFile(self):
        # Проверка до записи в двоичном режиме
        tests: str = self.test_str
        self.testClassFile.writeBinaryFile(tests.encode())
        self.testClassFile.appendBinaryFile(tests.encode())
        tests += tests
        self.assertEqual(tests.encode(), self.testClassFile.readBinaryFile())

    def test_readFile_Line(self):
        test_text = "123123\n3123133\n12312d1d12313"
        self.testClassFile.writeFile(test_text)
        self.assertEqual(self.testClassFile.readFile(2), "123123\n3123133\n")

    def test_readFileToResDict(self):
        self.testClassFile.writeFile("my name\nmy passwd\nmy token")
        res = self.testClassFile.readFileToResDict("name", "passwd", "token")
        self.assertEqual(res, {'name': 'my name', 'passwd': 'my passwd', 'token': 'my token'})

    def test_searchFile(self):
        test_text = "Optional. If the number of \n bytes returned exceed the hint number, \n no more lines will be returned. Default value is  -1, which means all lines will be returned."
        self.testClassFile.writeFile(test_text)
        self.assertEqual(self.testClassFile.searchFile("more"), True)

    def test___init__QuackCommand(self):
        w = TxtFile("test.txt", mod="w", data="123123")
        r1 = TxtFile("test.txt", mod="r")
        self.assertEqual(r1.res, "123123")
        w = TxtFile("test.txt", mod="a", data="99")
        r2 = TxtFile("test.txt", mod="r")
        self.assertEqual(r2.res, "12312399")

    def __del__(self):
        self.testClassFile.deleteFile()


class test_File(unittest.TestCase):

    def __init__(self, methodName: str = ...) -> None:
        super().__init__(methodName)
        self.name_file = "test/test/data/test.txt"

    def test_route(self):
        self.testClassFile = TxtFile(self.name_file)
        self.testClassFile.writeFile("123")
        self.assertEqual(self.testClassFile.readFile(), "123")
        self.testClassFile.removeRoute()


class TestJson(unittest.TestCase):

    def __init__(self, methodName: str = ...) -> None:
        super().__init__(methodName)
        # Данные для теста
        self.testlist: List[List, Dict] = [
            [1, 2.1, -1, -2.1, "1", "\t", "Qwe", "Фыв"],
            {12: 2, 1: 1, 1.2: 1.3, 13: 1.2, 4.2: 1, -12: 1, 41: -23, -23.1: -2.2, -232.2: 1,
             "Qwe": 1, 15: "Qwe", -21: "Qwe", 12.3: "DewW", -11: "wasd", "quests": -123},
        ]
        self.name_file = "test.json"

    # Этот метод запускаетсья ПЕРЕД каждой функции теста
    def setUp(self) -> None:
        self.testClassJson = JsonFile(self.name_file)
        self.testClassJson.deleteFile()
        self.testClassJson.createFileIfDoesntExist()

    def test_sizeFile(self):
        # Проверка определение размера файла
        self.testClassJson.writeFile(self.testlist, sort_keys=False)
        self.assertEqual(self.testClassJson.sizeFile(), getsize(self.testClassJson.name_file))

    def test_deleteFile_and_checkExistenceFile(self):
        # Проверка удаление файла
        self.assertEqual(self.testClassJson.checkExistenceFile(), True)
        self.testClassJson.deleteFile()
        self.assertEqual(self.testClassJson.checkExistenceFile(), False)

    def test_writeJsonFile_and_readJsonFile(self):
        # Проверка записи в файл разных структур данных
        # List
        self.testClassJson.deleteFile()
        self.testClassJson.createFileIfDoesntExist()
        temples: List = self.testlist[0]
        self.testClassJson.writeFile(temples)
        self.assertEqual(temples, self.testClassJson.readFile())

        # Dict
        self.testClassJson.deleteFile()
        self.testClassJson.createFileIfDoesntExist()
        temples: Dict = {str(k): v for k, v in self.testlist[1].items()}  # все ключи должны быть типа str
        self.testClassJson.writeFile(temples)
        self.assertEqual(temples, self.testClassJson.readFile())

    def test_appendJsonListFile(self):
        # Проверка до записи в файл разных структур данных
        # List
        self.testClassJson.deleteFile()
        self.testClassJson.createFileIfDoesntExist()
        tempers: List = self.testlist[0]
        self.testClassJson.writeFile(tempers)
        self.testClassJson.appendFile(tempers)
        tempers += tempers
        self.assertEqual(tempers, self.testClassJson.readFile())

        # Dict
        self.testClassJson.deleteFile()
        self.testClassJson.createFileIfDoesntExist()
        tempers: Dict = {str(k): v for k, v in self.testlist[1].items()}  # все ключи должны быть типа str
        self.testClassJson.writeFile(tempers)
        self.testClassJson.appendFile(tempers)
        tempers.update(tempers)
        self.assertEqual(tempers, self.testClassJson.readFile())

    def __del__(self):
        self.testClassJson.deleteFile()


class TestCsvFile(unittest.TestCase):

    def setUp(self):
        self.cvs_file = CsvFile("test.csv")
        self.cvs_file.deleteFile()

    def test_init_(self):
        # Реакция на некорректное им файла
        self.assertRaises(ValueError, CsvFile, "test.txt")

    def test_writeFile_and_readFile(self):
        # Проверка записи и чтения данных cvs файла
        self.cvs_file.writeFile(
            [[1, 23, 41, 5],
             [21, 233, 46, 35],
             [13, 233, 26, 45],
             [12, 213, 43, 56]], FlagDataConferToStr=True, header=("Данныe", "Data", "Числа", "Num"))

        #  Тест на чтение Cvs файла
        self.assertEqual(self.cvs_file.readFile(),
                         [['Данныe', 'Data', 'Числа', 'Num'], ['1', '23', '41', '5'], ['21', '233', '46', '35'],
                          ['13', '233', '26', '45'], ['12', '213', '43', '56']])

        #  Тест на чтение cvs файла с убранами заголовками
        self.assertEqual(self.cvs_file.readFile(miss_get_head=True),
                         [['1', '23', '41', '5'], ['21', '233', '46', '35'],
                          ['13', '233', '26', '45'], ['12', '213', '43', '56']])

        # Тест на личит чтнеия
        self.assertEqual(self.cvs_file.readFile(limit=2),
                         [['Данныe', 'Data', 'Числа', 'Num'], ['1', '23', '41', '5']])

        #  Тест на привышающий лимит чтения
        self.assertEqual(self.cvs_file.readFile(limit=1123),
                         [['Данныe', 'Data', 'Числа', 'Num'], ['1', '23', '41', '5'], ['21', '233', '46', '35'],
                          ['13', '233', '26', '45'], ['12', '213', '43', '56']])

        #  Тест на чтение в обратном порядке
        self.assertEqual(self.cvs_file.readFileRevers(),
                         [['12', '213', '43', '56'], ['13', '233', '26', '45'], ['21', '233', '46', '35'],
                          ['1', '23', '41', '5'], ['Данныe', 'Data', 'Числа', 'Num']])

        # Тест на лимит чтени в обратном порядке
        self.assertEqual(self.cvs_file.readFileRevers(limit=2), [['12', '213', '43', '56'], ['13', '233', '26', '45']])

        #  Тест на привышающий лимит чтения в обратном порядке
        self.assertEqual(self.cvs_file.readFileRevers(limit=111),
                         [['12', '213', '43', '56'], ['13', '233', '26', '45'], ['21', '233', '46', '35'],
                          ['1', '23', '41', '5'], ['Данныe', 'Data', 'Числа', 'Num']])

        self.cvs_file.deleteFile()

    def test_appendFile(self):
        # проверка до записи в файл
        self.cvs_file.deleteFile()

        # Проверка записи с флагом FlagDataConferToStr
        self.cvs_file.writeFile(
            [[1, 23, 41, 5],
             [21, 233, 46, 35],
             [13, 233, 26, 45],
             [12, 213, 43, 56]], FlagDataConferToStr=True, header=("Данные", "Data", "Числа", "Num"))

        self.cvs_file.appendFile([['2323', '23233', '23']])

        self.assertEqual(self.cvs_file.readFile(),
                         [['Данные', 'Data', 'Числа', 'Num'], ['1', '23', '41', '5'], ['21', '233', '46', '35'],
                          ['13', '233', '26', '45'], ['12', '213', '43', '56'], ['2323', '23233', '23']])

    def test_ordinary(self):
        # Тест записи одномерного массива

        self.cvs_file.deleteFile()
        self.cvs_file.writeFile([123, 123, 222, 1, 312, 223, 2], FlagDataConferToStr=True)
        self.cvs_file.writeFile([123, 123, 222, 1, 2], FlagDataConferToStr=True)
        self.cvs_file.writeFile([123, 123, '222', 1], FlagDataConferToStr=True)
        self.cvs_file.writeFile([123, 222, 1, 2])

        self.cvs_file.appendFile([123, 123, 222, 1, 312, 223, 2], FlagDataConferToStr=True)
        self.cvs_file.appendFile([123, 123, 222, 1, 2], FlagDataConferToStr=True)
        self.cvs_file.appendFile([123, 123, '222', 1], FlagDataConferToStr=True)
        self.cvs_file.appendFile(['123', '222', '1', '2'])

        # Тест записи двумерного массива
        self.cvs_file.deleteFile()
        self.cvs_file.writeFile(
            [[123, 123, 222, 1, 312, 223, 2],
             [4123, 1233, 222, 1, 3312, 223, 2],
             ], FlagDataConferToStr=True)
        self.cvs_file.writeFile(
            [[123, 123, 222, 1, 312, 223, 2],
             [4123, 1233, '222', 1, 3312, 223, 2],
             ], FlagDataConferToStr=True)

        self.cvs_file.writeFile(
            [[123, 123, 222, 1, 312, 223, 2],
             [4123, 1233, 222, 1, 3312, 223, 2],
             ])

        self.cvs_file.appendFile(
            [[123, 123, 222, 1, 312, 223, 2],
             [4123, 1233, 222, 1, 3312, 223, 2],
             ], FlagDataConferToStr=True)
        self.cvs_file.appendFile(
            [[123, 123, 222, 1, 312, 223, 2],
             [4123, 1233, '222', 1, 3312, 223, 2],
             ], FlagDataConferToStr=True)

        self.cvs_file.appendFile(
            [[123, 123, 222, 1, 312, 223, 2],
             [4123, 1233, 222, 1, 3312, 223, 2],
             ])

        # Тест Записи Float
        self.cvs_file.writeFile([123.12, 123.43, 222.2, 1.5, 31.2, 22.3, 2.5], FlagDataConferToStr=True)
        self.assertEqual(
            self.cvs_file.readFile(),
            [['123.12', '123.43', '222.2', '1.5', '31.2', '22.3', '2.5']])

        # Тест записи комберированно
        self.cvs_file.writeFile([12, 123.43, 'Hello Привет', '1.5', 31.2, 22.3, 2.5], FlagDataConferToStr=True),
        self.assertEqual(
            self.cvs_file.readFile(),
            [['12', '123.43', 'Hello Привет', '1.5', '31.2', '22.3', '2.5']])

        # Тест Записи Float
        self.cvs_file.writeFile([123.12, 123.43, 222.2, 1.5, 31.2, 22.3, 2.5]),
        self.assertEqual(self.cvs_file.readFile(),
                         [['123.12', '123.43', '222.2', '1.5', '31.2', '22.3', '2.5']])

        #
        # Тест записи комберированно
        self.cvs_file.writeFile([12, 123.43, 'Hello Привет', '1.5', 31.2, 22.3, 2.5]),
        self.assertEqual(self.cvs_file.readFile(),
                         [['12', '123.43', 'Hello Привет', '1.5', '31.2', '22.3', '2.5']])

    def test_readFileAndFindDifferences(self):
        data_file = [['Халява: на IndieGala бесплатно отдают аркадный футбол FootLOL: Epic Fail League',
                      'https://playisgame.com/halyava/halyava-na-indiegala-besplatno-otdayut-arkadnyy-futbol-footlol-epic-fail-league/'],
                     ['Халява: в Steam бесплатно отдают головоломку Landing и платформер Inops',
                      'https://playisgame.com/halyava/halyava-v-steam-besplatno-otdayut-golovolomku-landing-i-platformer-inops/'],
                     ['Халява: в For Honor можно играть бесплатно на выходных',
                      'https://playisgame.com/halyava/halyava-v-for-honor-mozhno-igrat-besplatno-na-vyhodnyh/'],
                     ['Халява: в сплатно раздают классические квесты Syberia I и Syberia II',
                      'https://playisgame.com/halyava/halyava-v-gog-besplatno-razdayut-klassicheskie-kvesty-syberia-i-i-syberia-ii/'],
                     ['Халява: о отдают музыкальный платформер Symphonia',
                      'https://playisgame.com/halyava/halyava-v-gog-besplatno-otdayut-muzykalnyy-platformer-symphonia/'],
                     ['Халява: на IndieGala бесплатно отдают Defense of Roman Britain в жанре защиты башень',
                      'https://playisgame.com/halyava/halyava-na-indiegala-besplatno-otdayut-defense-of-roman-britain-v-zhanre-zaschity-bashen/'],
                     ['Халява: в GOG можно бесплатно забрать подарки в честь WitcherCon',
                      'https://playisgame.com/halyava/haljava-v-gog-mozhno-besplatno-zabrat-podarki-v-chest-witchercon/'],
                     [
                         'Халява: в EGS бесплатно отдают симулятор Bridge Constructor: The Walking Dead и стратегию Ironcast',
                         'https://playisgame.com/halyava/haljava-v-egs-besplatno-otdajut-simuljator-bridge-constructor-the-walking-dead-i-strategiju-ironcast/'],
                     ['Халява: в GOG стартовала бесплатная раздача коллекции Shadowrun Trilogy',
                      'https://playisgame.com/halyava/khalyava-v-gog-startovala-besplatnaya-razdacha-kollektsii-shadowrun-trilogy/'],
                     ['Халява: в Hell Let Loose можно играть бесплатно на выходных',
                      'https://playisgame.com/halyava/khalyava-v-hell-let-loose-mozhno-igrat-besplatno-na-vykhodnykh/'],
                     ['Халява: в EGS бесплатно раздают Horizon Chase Turbo и Sonic Mania',
                      'https://playisgame.com/halyava/khalyava-v-egs-besplatno-razdayut-horizon-chase-turbo-i-sonic-mania/']]
        new_data = [['Халява: на IndieGala бесплатно отдают аркадный футбол FootLOL: Epic Fail League',
                     'https://playisgame.com/halyava/halyava-na-indiegala-besplatno-otdayut-arkadnyy-futbol-footlol-epic-fail-league/'],
                    ['Халява: в Steam бесплатно отдают головоломку Landing и платформер Inops',
                     'https://playisgame.com/halyava/halyava-v-steam-besplatno-otdayut-golovolomku-landing-i-platformer-inops/'],
                    ['Халява: в For Honor можно играть бесплатно на выходных',
                     'https://playisgame.com/halyava/halyava-v-for-honor-mozhno-igrat-besplatno-na-vyhodnyh/'],
                    ['Халява: ОШИБКА классические квесты Syberia I и Syberia II',
                     'https://playisgame.com/halyava/halyava-v-gog-besplatno-razdayut-klassicheskie-kvesty-syberia-i-i-syberia-ii/'],
                    ['Халява: о отдают музыкальный платформер Symphonia',
                     'https://playisgame.com/halyava/halyava-v-gog-besplatno-otdayut-muzykalnyy-platformer-symphonia/'],
                    ['Халява: на IndieGala ERROR отдают Defense of Roman Britain в жанре защиты башень',
                     'https://playisgame.com/halyava/halyava-na-indiegala-besplatno-otdayut-defense-of-roman-britain-v-zhanre-zaschity-bashen/'],
                    ['Халява: в GOG можно бесплатно забрать подарки в честь WitcherCon',
                     'https://playisgame.com/halyava/haljava-v-gog-mozhno-besplatno-zabrat-podarki-v-chest-witchercon/'],
                    [
                        'Халява: в EGS бесплатно отдают симулятор Bridge Constructor: The Walking Dead и стратегию Ironcast',
                        'https://playisgame.com/halyava/haljava-v-egs-besplatno-otdajut-simuljator-bridge-constructor-the-ERROR-dead-i-strategiju-ironcast/'],
                    ['Халява: в GOG стартовала бесплатная раздача коллекции Shadowrun Trilogy',
                     'https://playisgame.com/halyava/khalyava-v-gog-startovala-besplatnaya-razdacha-kollektsii-shadowrun-trilogy/'],
                    ['Халява: в Hell Let Loose можно играть бесплатно на выходных',
                     'https://playisgame.com/halyava/khalyava-v-hell-let-loose-mozhno-igrat-besplatno-na-vykhodnykh/'],
                    ['Халява: в EGS бесплатно раздают Horizon Chase Turbo и Sonic Mania',
                     'https://playisgame.com/halyava/khalyava-v-egs-besplatno-razdayut-horizon-chase-turbo-i-sonic-mania/']]

        tmpList = []
        self.cvs_file.writeFile(data_file, header=("Имя акции", "Ссылка"))
        self.cvs_file.readFileAndFindDifferences(new_data, tmpList.append)
        self.assertEqual(tmpList, [['Халява: ОШИБКА классические квесты Syberia I и Syberia II',
                                    'https://playisgame.com/halyava/halyava-v-gog-besplatno-razdayut-klassicheskie-kvesty-syberia-i-i-syberia-ii/'],
                                   ['Халява: на IndieGala ERROR отдают Defense of Roman Britain в жанре защиты башень',
                                    'https://playisgame.com/halyava/halyava-na-indiegala-besplatno-otdayut-defense-of-roman-britain-v-zhanre-zaschity-bashen/'],
                                   [
                                       'Халява: в EGS бесплатно отдают симулятор Bridge Constructor: The Walking Dead и стратегию Ironcast',
                                       'https://playisgame.com/halyava/haljava-v-egs-besplatno-otdajut-simuljator-bridge-constructor-the-ERROR-dead-i-strategiju-ironcast/']])

    def __del__(self):
        self.cvs_file.deleteFile()


class TestPickleFile(unittest.TestCase):
    def __init__(self, methodName: str = ...) -> None:
        super().__init__(methodName)
        self.name_file = "test_pickle.pkl"

    def setUp(self):
        self.pk = PickleFile(self.name_file)
        self.pk.deleteFile()

    def test_writeFile_and_readFile(self):
        # Проверка записи данных
        test_data = [
            (1, 2, 3, 4),
            [12, 23, 221],
            ["1231", 12, (2, 22)],
            {213123, 123213},
            {'s1': '213'},
        ]

        for td in test_data:
            self.pk.writeFile(td)
            self.assertEqual(self.pk.readFile(), td)
            self.pk.deleteFile()

        self.pk.deleteFile()

    def test_appendFile(self):
        test_data = [1, 2, 3, 4]
        new_data = [98, 678, 88]
        self.pk.writeFile(test_data)
        self.pk.appendFile(new_data)
        test_data += new_data
        self.assertEqual(self.pk.readFile(), test_data)


if __name__ == '__main__':
    unittest.main()



from abc import abstractmethod
from os import makedirs, remove, mkdir
from os.path import abspath, dirname, exists, getsize
from shutil import rmtree
from typing import Any, Callable

T_ConcatData = list[str | int | float] | list[list[str | int | float]] | dict | set


class BaseFile:
    """
    Функционал:
        - удаление файла
        - проверка существования файла
        - проверка существования файла и если его нет то создание
        - путь к файлу
        - размер файла
        - проверка разрешения открытия файла
    """
    __slots__ = "name_file"

    def __init__(self, name_file: str):
        self.name_file: str = name_file
        self.createFileIfDoesntExist()

    def createFileIfDoesntExist(self):
        # Создать файл если его нет
        if not exists(self.name_file):
            tmp_ = dirname(self.name_file)
            if tmp_:  # Если задан путь из папок
                makedirs(tmp_)  # Создаем путь из папок
                open(self.name_file, "w").close()
            else:  # Если указано только имя файла без папок
                open(self.name_file, "w").close()

    def checkExistenceFile(self) -> bool:  # +
        # Проверить существование файла
        return True if exists(self.name_file) else False

    def deleteFile(self):  # +
        # Удаление файла
        if self.checkExistenceFile():
            remove(self.route())

    def sizeFile(self) -> int:  # +
        # Размер файла в байтах
        return getsize(self.name_file)

    def route(self) -> str:  # +
        # Путь к файлу
        return abspath(self.name_file)

    def createRoute(self):
        tmp_route: str = ""
        for folder_name in self.name_file.split('/')[:-1]:
            tmp_route += folder_name
            mkdir(tmp_route)
            tmp_route += '/'

    def removeRoute(self):
        rmtree(self.name_file.split('/')[1])

    @abstractmethod
    def readFile(self, *arg) -> Any:
        ...

    @abstractmethod
    def writeFile(self, arg: Any):
        ...

    @abstractmethod
    def appendFile(self, arg: Any):
        ...


def ConcatData(callback: Callable, file_data: T_ConcatData, new_data: T_ConcatData):
    """
    Объединить два переменных одинакового типа
    @param new_data:
    @param file_data:
    @param callback: Вызовется при успешной проверки типов
    """

    if type(new_data) == type(new_data):
        match new_data:
            case list():
                file_data.extend(new_data)
            case tuple():
                file_data += new_data
            case dict() | set():
                file_data.update(new_data)
            case _:
                raise TypeError("Не поддерживаемый тип")
        callback(file_data)
    else:
        raise TypeError("Тип данных в файле и тип входных данных различны")


if __name__ == '__main__':
    pass



from os.path import splitext
from pickle import load, dump
from typing import Any, Union

from .base_file import BaseFile


class PickleFile(BaseFile):
    # https://docs.python.org/3/library/pickle.html
    def __init__(self, name_file: str):

        if splitext(name_file)[1] != ".pkl":  # Проверяем расширение файла
            raise ValueError("Файл должен иметь расширение .pkl")

        BaseFile.__init__(self, name_file)

    def writeFile(self, data: Any, *, protocol: int = 3):
        # Сериализовать и записать данные в файл
        with open(self.name_file, "wb") as _pickFile:
            dump(data, _pickFile, protocol=protocol)

    def readFile(self, **kwargs) -> Any:
        # Прочитать и десериализация данные из файла
        with open(self.name_file, "rb") as _pickFile:
            return load(_pickFile)

    def appendFile(self, data: Union[tuple, list, dict, set], *, protocol: int = 3):
        tmp_data = self.readFile()

        if type(data) == type(tmp_data):  # Входные данные должны быть такого же типа, что и в файле

            # List
            if type(data) == list:
                tmp_data.extend(data)
                self.writeFile(tmp_data, protocol=protocol)

            # Tuple
            elif type(data) == tuple:
                self.writeFile(tmp_data + data, protocol=protocol)

            # Dict Set
            elif type(data) == dict or type(data) == set:
                tmp_data.update(data)
                self.writeFile(tmp_data, protocol=protocol)

        else:
            raise TypeError("Тип данных в файле и тип входных данных различны")



from collections import namedtuple
from typing import Callable, Any

# Необходимо переопределить
CONNECT: Callable | object | None = None
ERROR: BaseException | None = None
MUTABLE_COMMAND_A: Callable[[Any, str, tuple | dict | list], Any] | None = None
READ_COMMAND_A: Callable[[Any, str, tuple | dict | list], Any] | None = None
# Переопределится в функции `Config`
SETTINGS_DB: dict | None = None


###########################################
def dictfetchall(cursor) -> list[dict[str, Any]]:
    """
    Вернуть в виде словаря
    """
    columns = [col[0] for col in cursor.description]
    return [
        dict(zip(columns, row))
        for row in cursor.fetchall()
    ]


def namedtuplefetchall(cursor) -> list[namedtuple]:
    """
    Вернуть в виде именовано го картежа
    """
    desc = cursor.description
    nt_result = namedtuple('_', [col[0] for col in desc])
    return [nt_result(*row) for row in cursor.fetchall()]


def getDataFromType(_cursor: Any, tdata: str) -> Any:
    """
    Вызвать функцию сериализатор на основе `tdata`
    """
    match tdata:
        case "d":
            return dictfetchall(_cursor)
        case "o":
            # Первый элемент
            return _cursor.fetchone()
        case "a":
            return _cursor.fetchall()
        case "n" | None:
            return namedtuplefetchall(_cursor)



from abc import abstractstaticmethod
from hashlib import sha512


class UserNameDontExistsError(Exception):
    def __str__(self):
        return "Пользователь с таким `user name` не существует"


def hashPassword(password: str) -> str:
    """
    Захеширвоать пароль
    """
    return sha512(password.encode('utf-8')).hexdigest()


class BaseSchema:
    """
    Базовая SQL модель
    """

    @abstractstaticmethod
    def create_tabel():
        """
        Создать таблицу в БД
        """
        ...



import unittest
from os import path, listdir
from os.path import getsize

from mg_file.sqllite_orm import *


class TestSqlLite(unittest.TestCase):

    def __init__(self, methodName: str = ...) -> None:
        super().__init__(methodName)
        self.name_db = "test.db"
        self.name_table = "stocks"

    def setUp(self) -> None:
        self.sq = SqlLiteQrm(self.name_db)
        self.sq.DeleteTable(self.name_table)

    def test_error_name(self):
        # Проверка на неправильные имена базы данных
        self.assertRaises(NameError, SqlLiteQrm, 'example.txt')
        self.assertRaises(NameError, SqlLiteQrm, 'example')
        self.assertRaises(NameError, SqlLiteQrm, 'ex.am.pl.et')

    def test_ExecuteTable_dict(self):
        # Провекра дополнительынх параметоров к созданию таблицы
        # Провекра записи через dict имен
        test_header = {"id": PrimaryKey(int),
                       "name": toTypeSql(str),
                       "old": NotNullDefault(int, 5),
                       "salary": NotNull(float)}
        self.sq.CreateTable(self.name_table, test_header)

        self.assertEqual(self.sq.header_table[self.name_table],
                         {'id': 'INTEGER PRIMARY KEY', 'name': 'TEXT', 'old': 'INTEGER NOT NULL DEFAULT 5',
                          'salary': 'REAL NOT NULL'})
        self.sq.ExecuteTable(self.name_table, {"id": 1, "name": "Anton", "old": 30, "salary": 3000.11})
        self.sq.ExecuteTable(self.name_table, {"id": 2, "name": "Katy", "old": 22, "salary": 3200.23})
        self.assertEqual(self.sq.GetTable(self.name_table), [(1, 'Anton', 30, 3000.11),
                                                             (2, 'Katy', 22, 3200.23)])
        self.sq.DeleteTable(self.name_table)

        # Проверка записи с AUTOINCREMENT
        test_header = {"id": PrimaryKeyAutoincrement(int),
                       "name": toTypeSql(str),
                       "old": NotNullDefault(int, 5),
                       "salary": NotNull(float)}

        self.sq.CreateTable(self.name_table, test_header)
        self.assertEqual(self.sq.header_table[self.name_table],
                         {'id': 'INTEGER PRIMARY KEY AUTOINCREMENT', 'name': 'TEXT',
                          'old': 'INTEGER NOT NULL DEFAULT 5', 'salary': 'REAL NOT NULL'})
        self.sq.ExecuteTable(self.name_table, {"name": "Anton", "old": 30, "salary": 3000.33})
        self.sq.ExecuteTable(self.name_table, {"name": "Katy", "old": 22, "salary": 3200.54})
        self.assertEqual(self.sq.GetTable(self.name_table), [(1, 'Anton', 30, 3000.33), (2, 'Katy', 22, 3200.54)])
        self.sq.DeleteTable(self.name_table)

        # Проверка на неправильное имя столбца в передаче параметров dict
        test_header = {"id": PrimaryKey(int),
                       "name": toTypeSql(str),
                       "old": NotNullDefault(int, 5),
                       "salary": NotNull(float)}
        self.sq.CreateTable(self.name_table, test_header)
        self.assertRaises(IndexError, self.sq.ExecuteTable, self.name_table,
                          {"id": 1, "ERORRRRRRRRR": "Anton", "ol222d": 30, "salary": 3000})
        self.assertRaises(IndexError, self.sq.ExecuteTable, self.name_table,
                          {"id": 2, "ERORRR123132RRRRRR": "Katy", "old": 22, "salary": 3200})
        self.sq.DeleteTable(self.name_table)

    def test_ExecuteManyTable(self):
        # Проверека запси list в Бд
        test_header = {"id": PrimaryKey(int),
                       "name": toTypeSql(str),
                       "old": NotNullDefault(int, 5),
                       "salary": NotNull(float)}
        test_data_list = [
            (1, "Denis", 13, 23232.1223),
            (2, "Enis", 123, 5656.123),
            (3, "Renyi", 133, 365.4),
            (4, "Tennis", 132, 436.123),
        ]
        self.sq.CreateTable(self.name_table, test_header)
        self.sq.ExecuteManyTable(self.name_table, test_data_list)
        self.assertEqual(self.sq.GetTable(self.name_table), test_data_list)

    def test_CreateTable(self):
        # Првоерка создания таблицы
        test_header = {"date": toTypeSql(str), "trans": toTypeSql(str), "symbol": toTypeSql(str),
                       "qty": toTypeSql(float), "price": toTypeSql(float)}
        self.sq.CreateTable(self.name_table, test_header)

        self.assertEqual(self.sq.header_table[self.name_table],
                         {'date': 'TEXT', 'trans': 'TEXT', 'symbol': 'TEXT', 'qty': 'REAL', 'price': 'REAL'})
        self.sq.DeleteTable(self.name_table)

        # Тест записи заголовка в виде SQL запроса
        self.sq.CreateTable(self.name_table, "(date TEXT, trans TEXT, symbol TEXT, qty REAL, price REAL)")
        self.assertEqual(self.sq.header_table[self.name_table],
                         {'date': 'TEXT', 'trans': 'TEXT', 'symbol': 'TEXT', 'qty': 'REAL', 'price': 'REAL'})

        self.sq.DeleteTable(self.name_table)

        # С доп параметрами
        self.sq.CreateTable(self.name_table, "(qty REAL PRIMARY KEY, date TEXT, trans TEXT, symbol TEXT, price REAL)")
        self.assertEqual(self.sq.header_table[self.name_table],
                         {'date': 'TEXT', 'trans': 'TEXT', 'symbol': 'TEXT', 'qty': 'REAL PRIMARY KEY',
                          'price': 'REAL'})

        self.sq.DeleteTable(self.name_table)

    def test_error_CreateTable(self):
        # Проверка создания таблицы с неправильными данными
        self.assertRaises(TypeError, toTypeSql, list)
        # # Если изменить test_header то нужно переписать test_header на sql запрос
        # self.assertRaises(TypeError, self.sq.CreateTable, self.name_table,
        #                   "(date text, trans text, symbol int, qty real, price real)")
        # Провекра Двойного создания Primary Key
        test_header = {"id": PrimaryKeyAutoincrement(int),
                       "url": PrimaryKey(int),
                       "name": toTypeSql(str),
                       "old": NotNullDefault(int, 5),
                       "salary": NotNull(float)}
        self.assertRaises(sqlite3.OperationalError, self.sq.CreateTable, self.name_table, test_header)

    def test_ExecuteTable_and_GetTable(self):
        # Првоерка коректности записи данных в таблицу
        # Через dict

        test_header = {"date": toTypeSql(str), "trans": toTypeSql(str), "symbol": toTypeSql(str),
                       "qty": toTypeSql(float),
                       "price": toTypeSql(float)}
        test_data = ('2006-01-05', 'BUY', 'RAT', 100, 35.14)
        self.sq.CreateTable(self.name_table, test_header)
        self.sq.ExecuteTable(self.name_table, test_data)
        self.assertEqual(self.sq.GetTable(self.name_table)[0], test_data)
        self.sq.DeleteTable(self.name_table)
        # Через SQL запрос
        self.sq.CreateTable(self.name_table, {"id_like": toTypeSql(int)})
        self.sq.ExecuteTable(self.name_table, 2323)
        self.assertEqual(self.sq.GetTable(self.name_table)[0][0], 2323)
        self.sq.DeleteTable(self.name_table)

    def test_ExecuteTable_Blob(self):
        # Проверка записи BLOB через
        # Tuple
        test_header = {"str": toTypeSql(str), "int": toTypeSql(int), "float": toTypeSql(float),
                       "bytes": toTypeSql(bytes)}
        test_data = ("text", 123, 122.32, b"1011")
        self.sq.CreateTable(self.name_table, test_header)
        self.assertEqual(self.sq.header_table[self.name_table], test_header)
        self.sq.ExecuteTable(self.name_table, test_data, CheckBLOB=True)
        self.assertEqual(self.sq.GetTable(self.name_table)[0], test_data)
        self.sq.DeleteTable(self.name_table)
        # List
        self.sq.CreateTable(self.name_table, test_header)
        self.assertEqual(self.sq.header_table[self.name_table], test_header)
        self.sq.ExecuteTable(self.name_table, list(test_data), CheckBLOB=True)
        self.assertEqual(self.sq.GetTable(self.name_table)[0], test_data)
        self.sq.DeleteTable(self.name_table)

        # Проверка попытки записи типа BLOB через строку -> должны быть ошибка TypeError
        # test_header = {"str": toTypeSql(str), "int": toTypeSql(int), "float": toTypeSql(float), "bytes": toTypeSql(bytes)}
        test_data = "('text', '123', '122.32', '{0}')".format(b"0101")
        self.sq.CreateTable(self.name_table, "(str TEXT, int INTEGER, float REAL, bytes BLOB)")

        self.assertEqual(self.sq.header_table[self.name_table],
                         {'bytes': 'BLOB', 'float': 'REAL', 'int': 'INTEGER', 'str': 'TEXT'})

        self.assertRaises(TypeError, self.sq.ExecuteTable, (self.name_table, test_data))
        self.sq.DeleteTable(self.name_table)

        # ExecuteManyTable запись BLOB и проврека CheckBLOB
        self.sq.CreateTable(self.name_table, {
            'car_id': PrimaryKeyAutoincrement(int),
            "model": toTypeSql(str),
            "price": toTypeSql(bytes)
        })
        car = [
            ["Audi", b'432'],
            ["Maer", b'424'],
            ["Skoda", b"122"]
        ]
        self.sq.ExecuteManyTable(self.name_table, car, head_data=("model", "price"), CheckBLOB=True)
        self.assertEqual(self.sq.GetTable(self.name_table),
                         [(1, 'Audi', b'432'), (2, 'Maer', b'424'), (3, 'Skoda', b'122')])

    def test_error_ExecuteTable(self):
        # Проверка записи в таблицу неправильные данные
        test_header = {"date": toTypeSql(str), "trans": toTypeSql(str), "symbol": toTypeSql(str),
                       "qty": toTypeSql(float),
                       "price": toTypeSql(float)}
        self.sq.CreateTable(self.name_table, test_header)

        self.assertRaises(IndexError, self.sq.ExecuteTable, self.name_table,
                          ('2006-01-05', 'BUY', 'RAT', 100, 35.14, 100010001))  # Привышение длины tuple,list

        self.assertRaises(IndexError, self.sq.ExecuteTable, self.name_table,
                          ('2006-01-05', 'BUY', 35.14, 100010001))  # Маленькая длины tuple,list

    def test_ExecuteTable_sqlRequest(self):
        # Проверка добавления данных ExecuteTable через ReturnSqlRequest
        self.sq.CreateTable(self.name_table,
                            {"id": toTypeSql(int),
                             "old": toTypeSql(int)
                             })
        test_table: str = 'test_table'
        self.sq.CreateTable(test_table,
                            {"old": toTypeSql(int)})

        self.sq.ExecuteManyTable(self.name_table,
                                 [[11, 24],
                                  [22, 31],
                                  [2312, 312],
                                  [231, 68],
                                  [344, 187]])

        resSQL = Select(self.name_table, "id").Where("id < 30").Request

        self.sq.ExecuteTable(test_table, sqlRequest=resSQL)
        self.assertEqual(self.sq.GetTable(test_table), [(11,), (22,)])

    def test_DeleteDb(self):
        # Провекра удаления Бд
        test_header = {"date": toTypeSql(str), "trans": toTypeSql(str), "symbol": toTypeSql(str),
                       "qty": toTypeSql(float), "price": toTypeSql(float)}
        self.sq.CreateTable(self.name_table, test_header)
        self.assertEqual(path.exists(self.name_db), True)
        self.sq.DeleteDb()
        self.assertEqual(path.exists(self.name_db), False)

    def test_total(self):
        # Проверка записи в БД списка директории
        self.sq.CreateTable(self.name_table, {"name_file": toTypeSql(str), "size_file": toTypeSql(int)})
        for nf in listdir("D:"):
            if len(nf.split(".")) == 2 and nf.split(".")[1] == "py":
                self.sq.ExecuteTable(self.name_table, (nf, getsize(nf)))
        self.assertEqual(self.sq.GetTable(self.name_table), [(nf, getsize(nf)) for nf in listdir("D:") if
                                                             len(nf.split(".")) == 2 and nf.split(".")[1] == "py"])

    def test_Search(self):
        # Проврека функции поиска Search obj_Select SqlDataClass

        # Проверка Формирование запросов
        self.assertEqual(Select("table", "id", "name", "address").Request, 'SELECT id, name, address FROM table')
        self.assertEqual(Select("table", "*").Request, 'SELECT * FROM table')
        self.assertEqual(Select("table", "id", "name", "address").Where("id == 10").Request,
                         'SELECT id, name, address FROM table WHERE id == 10')
        self.assertEqual(Select("table", "id", "name", "address").Where("id == 10").Limit(10).Request,
                         'SELECT id, name, address FROM table WHERE id == 10 LIMIT 10 OFFSET 0')
        self.assertEqual(Select("table", "id", "name", "address").Where("id == 10").Limit(10, 1).Request,
                         'SELECT id, name, address FROM table WHERE id == 10 LIMIT 10 OFFSET 1')
        self.assertEqual(Select("table", "id", "name", "address").Where("id == 10").GroupBy("id").Request,
                         'SELECT id, name, address FROM table WHERE id == 10 GROUP BY id')
        self.assertEqual(Select("table", "id", "name", "address").Where("id == 10").GroupBy("id", "name").Request,
                         'SELECT id, name, address FROM table WHERE id == 10 GROUP BY id, name')
        self.assertEqual(
            Select("table", "id", "name", "address").Where("id >= 10").GroupBy("id", "name").Limit(10).Request,
            'SELECT id, name, address FROM table WHERE id >= 10 GROUP BY id, name LIMIT 10 OFFSET 0')
        self.assertEqual(Select("table", "id", "name", "address").Where("id != 10").OrderBy("id").Request,
                         'SELECT id, name, address FROM table WHERE id != 10 ORDER BY id ASC')
        self.assertEqual(Select("table", "id", "name", "address").Where("id <= 10").OrderBy("id").Limit(5).Request,
                         'SELECT id, name, address FROM table WHERE id <= 10 ORDER BY id ASC LIMIT 5 OFFSET 0')
        self.assertEqual(
            Select("table", "id", "name", "address").Join("new_table", "id.table == id.new_table").Where(
                "id <= 10").OrderBy("id").Limit(5).Request,
            'SELECT id, name, address FROM table INNER JOIN new_table ON id.table == id.new_table WHERE id <= 10 ORDER BY id ASC LIMIT 5 OFFSET 0')

        self.assertRaises(ValueError, Select, "*")

        # Прворека поиска в тестовой бд
        self.sq.CreateTable(self.name_table,
                            {"id": PrimaryKeyAutoincrement(int), "name": toTypeSql(str), "old": toTypeSql(int),
                             "sex": NotNullDefault(str, "_")})
        self.sq.ExecuteManyTableDict(self.name_table,
                                     [{"name": "Denis", "old": 21},
                                      {"name": "Katy", "old": 21, "sex": 1},
                                      {"name": "Svetha", "old": 24}]
                                     )

        self.assertEqual(self.sq.Search(Select(self.name_table, "name").Where("old == 21")),
                         [('Denis',), ('Katy',)])

        self.assertEqual(self.sq.Search(Select(self.name_table, "name", "sex").Where("old == 21")),
                         [('Denis', '_'), ('Katy', '1')])

        self.assertEqual(self.sq.Search(Select(self.name_table, "*").Where("old == 21")),
                         [(1, 'Denis', 21, '_'), (2, 'Katy', 21, '1')])

        self.assertEqual(self.sq.Search(Select(self.name_table, "name").Where("old <= 21")),
                         [('Denis',), ('Katy',)])

        self.assertEqual(self.sq.Search(Select(self.name_table, "name").Where("old < 21")), [])

        self.assertEqual(self.sq.Search(Select(self.name_table, "name").Where("old > 21")), [('Svetha',)])
        self.assertEqual(self.sq.Search(Select(self.name_table, "name").Where("old >= 21")),
                         [('Denis',), ('Katy',), ('Svetha',)])

        # Проверка FlagPrint Проверять вручнкю
        # self.sq.Search(obj_Select(self.name_table, "*"), FlagPrint=10)
        # self.sq.Search(obj_Select(self.name_table, "*").Where("old > 21"), FlagPrint=10)
        # self.sq.Search(obj_Select(self.name_table, "name"), FlagPrint=12)
        # self.sq.Search(obj_Select(self.name_table, "name").Where("old > 21"), FlagPrint=9)
        # self.sq.Search(obj_Select(self.name_table, "name", "old"), FlagPrint=11)

        self.sq.DeleteTable(self.name_table)

    def test_SearchColumn(self):
        # Проверка алгоритма поиска в таблицы
        self.sq.CreateTable(self.name_table,
                            {"id": PrimaryKeyAutoincrement(int), "name": toTypeSql(str), "old": toTypeSql(int),
                             "sex": NotNullDefault(str, "_")})
        self.sq.ExecuteManyTableDict(self.name_table,
                                     [{"name": "Denis", "old": 21},
                                      {"name": "Katy", "old": 21, "sex": 1},
                                      {"name": "Svetha", "old": 24}]
                                     )

        self.assertEqual(self.sq.Search(Select(self.name_table, "name").Where("old == 21")),
                         [('Denis',), ('Katy',)])

        self.assertEqual(self.sq.Search(Select(self.name_table, "name", "sex").Where("old == 21")),
                         [('Denis', '_'), ('Katy', '1')])

        self.assertEqual(self.sq.Search(Select(self.name_table, "*").Where("old == 21")),
                         [(1, 'Denis', 21, '_'), (2, 'Katy', 21, '1')])

        self.assertEqual(self.sq.Search(Select(self.name_table, "name").Where("old <= 21")),
                         [('Denis',), ('Katy',)])

        self.assertEqual(self.sq.Search(Select(self.name_table, "name").Where("old < 21")), [])

        self.assertEqual(self.sq.Search(Select(self.name_table, "name").Where("old > 21")), [('Svetha',)])

        self.assertEqual(self.sq.Search(Select(self.name_table, "name").Where("old >= 21")),
                         [('Denis',), ('Katy',), ('Svetha',)])

        self.sq.DeleteTable(self.name_table)

        # Двойное условие SearchColumn
        self.sq.CreateTable(self.name_table,
                            {"id": PrimaryKeyAutoincrement(int), "name": toTypeSql(str), "old": toTypeSql(int),
                             "sex": NotNullDefault(str, "_")})
        self.sq.ExecuteManyTableDict(self.name_table, [{"name": "Denis", "old": 21},
                                                       {"name": "Katy", "old": 21, "sex": 1},
                                                       {"name": "Mush", "old": 21, "sex": 21},
                                                       {"name": "Patio", "old": 21, "sex": 21},
                                                       {"name": "Svetha", "old": 24}])

        self.assertEqual(self.sq.Search(Select(self.name_table, "name").Where("old == 21 and sex == 21")),
                         [('Mush',), ('Patio',)])  # И

        self.assertEqual(self.sq.Search(Select(self.name_table, "name").Where("old BETWEEN 10 and 21")),
                         [('Denis',), ('Katy',), ('Mush',), ('Patio',)])  # В пределах

        self.assertEqual(self.sq.Search(Select(self.name_table, "name").Where("old in (24,22)")),
                         [('Svetha',)])  # Содержиться В ()

        self.assertEqual(self.sq.Search(Select(self.name_table, "name").Where("old == 21 or sex == 1")),
                         [('Denis',), ('Katy',), ('Mush',), ('Patio',)])  # ИЛИ

        self.assertEqual(self.sq.Search(Select(self.name_table, "name").Where("old not in (21,24)")),
                         [])  # Приставка НЕ
        self.sq.DeleteTable(self.name_table)

        # Сортировка ORDER BY
        self.sq.CreateTable(self.name_table,
                            {"id": PrimaryKeyAutoincrement(int), "name": toTypeSql(str), "old": toTypeSql(int),
                             "sex": NotNullDefault(str, "_")})
        self.sq.ExecuteManyTableDict(self.name_table, [{"name": "Denis", "old": 21},
                                                       {"name": "Katy", "old": 221, "sex": 1},
                                                       {"name": "Mush", "old": 321, "sex": 21},
                                                       {"name": "Patio", "old": 231, "sex": 21},
                                                       {"name": "Svetha", "old": 24}])

        self.assertEqual(
            self.sq.Search(Select(self.name_table, "*").Where("old > 20").OrderBy("old")),
            [(1, 'Denis', 21, '_'),
             (5, 'Svetha', 24, '_'),
             (2, 'Katy', 221, '1'),
             (4, 'Patio', 231, '21'),
             (3, 'Mush', 321, '21')])

        # Сортировка ORDER BY ___ [DESC,ASC]

        self.assertEqual(
            self.sq.Search(Select(self.name_table, "*").Where("old > 20").OrderBy("id")),
            [(1, 'Denis', 21, '_'), (2, 'Katy', 221, '1'), (3, 'Mush', 321, '21'),
             (4, 'Patio', 231, '21'), (5, 'Svetha', 24, '_')])

        self.assertEqual(
            self.sq.Search(Select(self.name_table, "*").Where("old > 20").OrderBy("-id")),
            [(5, 'Svetha', 24, '_'), (4, 'Patio', 231, '21'), (3, 'Mush', 321, '21'),
             (2, 'Katy', 221, '1'), (1, 'Denis', 21, '_')])

        # LIMIT
        self.assertEqual(self.sq.Search(Select(self.name_table, "*").Where("old > 20").OrderBy("id").Limit(2)),
                         [(1, 'Denis', 21, '_'), (2, 'Katy', 221, '1')])  # До 2

        self.assertEqual(
            self.sq.Search(Select(self.name_table, "*").Where("old > 20").OrderBy("id").Limit(4, 2)),
            [(3, 'Mush', 321, '21'), (4, 'Patio', 231, '21'),
             (5, 'Svetha', 24, '_')])  # До 4 с интервалам 2

        self.assertEqual(
            Select(self.name_table, "*").Where("old > 20").OrderBy("id").Limit(4, 2).Request,
            'SELECT * FROM stocks WHERE old > 20 ORDER BY id ASC LIMIT 4 OFFSET 2')

    def test_ExecuteManyTable_and_sqlJOIN(self):
        # Проверка когда созданы ДВЕ таблицы и мы проверяем то что заголовки указаны правильно для заполнения

        self.sq.CreateTable(self.name_table, {"id": toTypeSql(int), "name": toTypeSql(str), "old": toTypeSql(int)})
        self.sq.ExecuteManyTable(self.name_table, [
            [0, "Denis", 21],
            [1, "Musha", 25],
            [2, "Dima", 33]
        ])
        # Проверка когда созданы ДВЕ таблицы и мы проверяем то что заголовки указаны правильно для заполнения

        self.sq.DeleteTable("new")
        self.sq.CreateTable("new", {"id": toTypeSql(int), "many": toTypeSql(int), "score": toTypeSql(int),
                                    "any": toTypeSql(float)})
        self.sq.ExecuteManyTable("new", [
            [0, 3323, 11, 1.1],
            [0, 21, 11, 1.1],
            [2, 223, 33, 1.1],
            [1, 21, 25, 1.1],
            [0, 3323, 11, 1.1],
            [2, 23, 33, 1.1],
            [1, 324, 25, 1.1],
            [0, 3323, 11, 1.1]
        ])

        self.assertEqual(self.sq.GetTable(self.name_table), [(0, 'Denis', 21), (1, 'Musha', 25), (2, 'Dima', 33)])
        self.assertEqual(self.sq.GetTable("new"),
                         [(0, 3323, 11, 1.1), (0, 21, 11, 1.1), (2, 223, 33, 1.1), (1, 21, 25, 1.1), (0, 3323, 11, 1.1),
                          (2, 23, 33, 1.1), (1, 324, 25, 1.1), (0, 3323, 11, 1.1)])

        # Провекра LIMIT у GetTable
        self.assertEqual(self.sq.GetTable("new", LIMIT=(3, 0)),  # Переделать лимит
                         [(0, 3323, 11, 1.1), (0, 21, 11, 1.1), (2, 223, 33, 1.1)])

        # Проверка sqlJOIN
        self.assertEqual(
            self.sq.Search(Select(self.name_table, f'{self.name_table}.name', "new.many").Join("new",
                                                                                               f"{self.name_table}.id =  new.id")),
            [('Denis', 21), ('Denis', 3323), ('Denis', 3323), ('Denis', 3323), ('Musha', 21),
             ('Musha', 324), ('Dima', 23), ('Dima', 223)])

        self.sq.DeleteTable(self.name_table)
        self.sq.DeleteTable("new")

        # Проверка записи list в ExecuteManyTable через второй агругемент с именем
        self.sq.CreateTable(self.name_table, {
            'car_id': PrimaryKeyAutoincrement(int),
            "model": toTypeSql(str),
            "price": toTypeSql(int)
        })

        car = [
            ("Audi", 432),
            ("Maer", 424),
            ("Skoda", 122)
        ]
        self.sq.ExecuteManyTable(self.name_table, car, head_data=("model", "price"))
        self.assertEqual(self.sq.GetTable(self.name_table), [(1, 'Audi', 432), (2, 'Maer', 424), (3, 'Skoda', 122)])

    def test_ExecuteManyTableDict(self):
        # Проверка записи ExecuteManyTableDict
        self.sq.CreateTable(self.name_table,
                            {"id": PrimaryKeyAutoincrement(int), "name": toTypeSql(str), "old": toTypeSql(int),
                             "sex": NotNullDefault(str, "_")})
        self.sq.ExecuteManyTableDict(self.name_table,
                                     [{"name": "Denis", "old": 21},
                                      {"name": "Katy", "old": 21, "sex": 1},
                                      {"name": "Svetha", "old": 24}]
                                     )
        self.assertEqual(self.sq.GetTable(self.name_table),
                         [(1, 'Denis', 21, '_'), (2, 'Katy', 21, '1'), (3, 'Svetha', 24, '_')])

    def test_ListTables_and___update_header_table(self):
        # Проверка получения списка таблиц
        test_header = {"id": PrimaryKey(int),
                       "name": toTypeSql(str),
                       "old": NotNullDefault(int, 5),
                       "salary": NotNull(float)}

        self.sq.CreateTable(self.name_table, test_header)

        test_header = {"id": PrimaryKey(int),
                       "film": toTypeSql(str),
                       "old": toTypeSql(int),
                       "salary": toTypeSql(float)}
        self.sq.CreateTable(self.name_table, test_header)
        self.sq.CreateTable("test1", test_header)

        test_header = {"id": PrimaryKey(int),
                       "name": toTypeSql(str),
                       "happy": NotNullDefault(int, 5),
                       "salary": toTypeSql(float)}
        self.sq.CreateTable(self.name_table, test_header)
        self.sq.CreateTable("test2", test_header)

        self.assertEqual(self.sq.ListTables(), ['stocks', 'test1', 'test2'])

        # Проверка __update_header_table для новой таблицы
        sq_test = SqlLiteQrm(self.name_db)
        self.assertEqual(sq_test.ListTables(), ['stocks', 'test1', 'test2'])
        self.assertEqual(sq_test.header_table['test1'], {"id": PrimaryKey(int),
                                                         "film": toTypeSql(str),
                                                         "old": toTypeSql(int),
                                                         "salary": toTypeSql(float)})

    def test_GetColumn(self):
        # Проверка получения столбца
        self.sq.CreateTable(self.name_table,
                            {"id": PrimaryKeyAutoincrement(int), "name": toTypeSql(str), "old": toTypeSql(int),
                             "sex": NotNullDefault(str, "_")})
        self.sq.ExecuteManyTableDict(self.name_table,
                                     [{"name": "Denis", "old": 21},
                                      {"name": "Katy", "old": 21, "sex": 1},
                                      {"name": "Svetha", "old": 24}]
                                     )

        self.assertEqual(self.sq.GetColumn(self.name_table, "name"), ['Denis', 'Katy', 'Svetha'])
        self.assertEqual(self.sq.GetColumn(self.name_table, "old"), [21, 21, 24])
        self.assertEqual(self.sq.GetColumn(self.name_table, "old", LIMIT=(2, 0)), [21, 21])

    def tests_UpdateColumne(self):
        # Проверка обновления данных
        self.sq.CreateTable(self.name_table,
                            {"id": PrimaryKeyAutoincrement(int), "name": toTypeSql(str), "old": toTypeSql(int),
                             "sex": NotNullDefault(str, "_")})
        self.sq.ExecuteManyTableDict(self.name_table, [{"name": "Denis", "old": 21},
                                                       {"name": "Katy", "old": 221, "sex": 1},
                                                       {"name": "Mush", "old": 321, "sex": 21},
                                                       {"name": "Patio", "old": 231, "sex": 21},
                                                       {"name": "Svetha", "old": 24}])

        self.sq.UpdateColumne(self.name_table, 'old', 99)
        self.assertEqual(self.sq.GetTable(self.name_table),
                         [(1, 'Denis', 99, '_'), (2, 'Katy', 99, '1'), (3, 'Mush', 99, '21'),
                          (4, 'Patio', 99, '21'), (5, 'Svetha', 99, '_')])
        self.sq.DeleteTable(self.name_table)

        # Проверка обновления данных с WHERE
        self.sq.CreateTable(self.name_table,
                            {"id": PrimaryKeyAutoincrement(int), "name": toTypeSql(str), "old": toTypeSql(int),
                             "sex": NotNullDefault(str, "_")})
        self.sq.ExecuteManyTableDict(self.name_table, [{"name": "Denis", "old": 21},
                                                       {"name": "Katy", "old": 221, "sex": 1},
                                                       {"name": "Mush", "old": 321, "sex": 21},
                                                       {"name": "Patio", "old": 231, "sex": 21},
                                                       {"name": "Svetha", "old": 24}])

        self.sq.UpdateColumne(self.name_table, 'old', 99, "sex > 21")
        self.assertEqual(self.sq.GetTable(self.name_table),
                         [(1, 'Denis', 99, '_'), (2, 'Katy', 221, '1'), (3, 'Mush', 321, '21'),
                          (4, 'Patio', 231, '21'), (5, 'Svetha', 99, '_')])
        self.sq.DeleteTable(self.name_table)

        # Проверка обновления данных с WHERE и LIKE
        self.sq.CreateTable(self.name_table,
                            {"id": PrimaryKeyAutoincrement(int), "name": toTypeSql(str), "old": toTypeSql(int),
                             "sex": NotNullDefault(str, "_")})
        self.sq.ExecuteManyTableDict(self.name_table, [{"name": "Denis", "old": 21},
                                                       {"name": "Katy", "old": 221, "sex": 1},
                                                       {"name": "Mush", "old": 321, "sex": 21},
                                                       {"name": "Patio", "old": 231, "sex": 21},
                                                       {"name": "Pvetha", "old": 24}])

        self.sq.UpdateColumne(self.name_table, 'old', 99, "name LIKE 'P%'")
        self.assertEqual(self.sq.GetTable(self.name_table),
                         [(1, 'Denis', 21, '_'), (2, 'Katy', 221, '1'), (3, 'Mush', 321, '21'),
                          (4, 'Patio', 99, '21'), (5, 'Pvetha', 99, '_')])
        self.sq.DeleteTable(self.name_table)

        # Проверка изменения нескольких стобцов
        self.sq.CreateTable(self.name_table,
                            {"id": PrimaryKeyAutoincrement(int), "name": toTypeSql(str), "old": toTypeSql(int),
                             "sex": NotNullDefault(str, "_")})
        self.sq.ExecuteManyTableDict(self.name_table, [{"name": "Denis", "old": 21},
                                                       {"name": "Katy", "old": 221, "sex": 1},
                                                       {"name": "Mush", "old": 321, "sex": 21},
                                                       {"name": "Patio", "old": 231, "sex": 21},
                                                       {"name": "Pvetha", "old": 24}])

        self.sq.UpdateColumne(self.name_table, ['old', 'sex'], [99, 88], "old > 21")
        self.assertEqual(self.sq.GetTable(self.name_table),
                         [(1, 'Denis', 21, '_'), (2, 'Katy', 99, '88'), (3, 'Mush', 99, '88'),
                          (4, 'Patio', 99, '88'), (5, 'Pvetha', 99, '88')])

        self.sq.DeleteTable(self.name_table)

    def test_DeleteLineTable(self):
        self.sq.CreateTable(self.name_table,
                            {"id": PrimaryKeyAutoincrement(int), "name": toTypeSql(str), "old": toTypeSql(int),
                             "sex": NotNullDefault(str, "_")})
        self.sq.ExecuteManyTableDict(self.name_table, [{"name": "Denis", "old": 21},
                                                       {"name": "Katy", "old": 221, "sex": 1},
                                                       {"name": "Mush", "old": 321, "sex": 21},
                                                       {"name": "Patio", "old": 231, "sex": 21},
                                                       {"name": "Pvetha", "old": 24}])
        self.sq.DeleteLineTable(self.name_table, "old > 25")
        self.assertEqual(self.sq.GetTable(self.name_table), [(1, 'Denis', 21, '_'), (5, 'Pvetha', 24, '_')])

    def test_AggregatingSearchColumne(self):
        # Тестирование агрегирующие функций и группировок
        self.sq.CreateTable(self.name_table,
                            {"id": PrimaryKeyAutoincrement(int), "name": toTypeSql(str), "old": toTypeSql(int),
                             "sex": NotNullDefault(str, "_")})
        self.sq.ExecuteManyTableDict(self.name_table, [{"name": "Denis", "old": 21},
                                                       {"name": "Katy", "old": 221, "sex": 1},
                                                       {"name": "Mush", "old": 321, "sex": 21},
                                                       {"name": "Patio", "old": 231, "sex": 21},
                                                       {"name": "Denis", "old": 24}])

        self.assertEqual(self.sq.Search(Select(self.name_table, CountSql("sex")).Where("old < 25")), [(2,)])

        self.assertEqual(self.sq.Search(Select(self.name_table, SumSql("old")).Where("old < 25")), [(45,)])

        self.assertEqual(self.sq.Search(Select(self.name_table, MaxSql("old")).Where("old < 25")), [(24,)])

        self.assertEqual(self.sq.Search(Select(self.name_table, MinSql("old")).Where("old < 25")), [(21,)])

        self.assertEqual(self.sq.Search(Select(self.name_table, AvgSql("old"))), [(163.6,)])

        # Проверка уникального столбца DISTINCT

        self.assertEqual(self.sq.Search(Select(self.name_table, CountSql("-name")).Where("old < 25")), [(1,)])
        # Проверка sqlGROUPBY
        self.assertEqual(
            self.sq.Search(Select(self.name_table, "name", SumSql("old")).GroupBy("name")),
            [('Denis', 45), ('Katy', 221), ('Mush', 321), ('Patio', 231)])

        self.assertEqual(self.sq.Search(Select(self.name_table, "*").GroupBy("sex", "name")),
                         [(2, 'Katy', 221, '1'), (3, 'Mush', 321, '21'), (4, 'Patio', 231, '21'),
                          (1, 'Denis', 21, '_')])

    def test_HeadTable_and___print_table(self):
        # Проверять на глаз
        self.sq.CreateTable(self.name_table, {"id": toTypeSql(int), "name": toTypeSql(str), "old": toTypeSql(int)})
        self.sq.HeadTable(self.name_table, 1)
        print(self.sq.HeadTable(self.name_table, 15))

    def test_Execute_ALL(self):
        # Проверка того что я не изменил строчку которая корректно подставляет имена SQL запросы
        # request += " ('{0}') VALUES ({1})".format("', '".join(self.header_table[name_table].keys()), res)

        self.sq.CreateTable(self.name_table, {'id_user': toTypeSql(int)})
        data = [[56654639], [236880983], [248579230], [248968841], [388967786], [410456140], [77708645], [132058482],
                [136393787], [144827581], [155823759], [171079623], [526765559]]
        self.sq.ExecuteManyTable(self.name_table, data)

        self.assertEqual(self.sq.GetTable(self.name_table),
                         [(56654639,), (236880983,), (248579230,), (248968841,), (388967786,), (410456140,),
                          (77708645,), (132058482,), (136393787,), (144827581,), (155823759,), (171079623,),
                          (526765559,)])

    def test_SaveDbToFile_ReadFileToDb(self):
        # # ExecuteManyTable запись BLOB и проверка CheckBLOB
        self.sq.CreateTable(self.name_table, {
            'car_id': PrimaryKeyAutoincrement(int),
            "model": toTypeSql(str),
            "price": toTypeSql(bytes)
        })
        car = [
            ["Audi", b'432'],
            ["Maer", b'424'],
            ["Skoda", b"122"]
        ]
        self.sq.ExecuteManyTable(self.name_table, car, head_data=("model", "price"), CheckBLOB=True)

        # тут появляеться запись
        # DELETE FROM "sqlite_sequence";INSERT INTO "sqlite_sequence" VALUES(\'stocks\',3);
        # Который вызвывает ошибку чтения данных из файла, функция ReadFileToDb обрабатывает эту ошибку
        # Если этой ошибку больше не будет то и функцию ReadFileToDb можно упростить
        # tmp = ""
        # with sqlite3.connect(self.name_db) as connection:
        #     for sql in connection.iterdump():
        #         tmp += sql
        # self.assertEqual(tmp,
        #                  'BEGIN TRANSACTION;DELETE FROM "sqlite_sequence";INSERT INTO "sqlite_sequence" VALUES(\'stocks\',3);CREATE TABLE stocks (car_id INTEGER PRIMARY KEY AUTOINCREMENT, model TEXT, price BLOB);INSERT INTO "stocks" VALUES(1,\'Audi\',X\'343332\');INSERT INTO "stocks" VALUES(2,\'Maer\',X\'343234\');INSERT INTO "stocks" VALUES(3,\'Skoda\',X\'313232\');COMMIT;')

        self.sq.SaveDbToFile("test_save.txt")
        self.sq.DeleteDb()
        self.sq.ReadFileToDb("test_save.txt")
        self.assertEqual(self.sq.GetTable(self.name_table),
                         [(1, 'Audi', b'432'), (2, 'Maer', b'424'), (3, 'Skoda', b'122')])

        tm = TxtFile("test_save.txt")
        tm.deleteFile()

    def __del__(self):
        self.sq.DeleteDb()


if __name__ == '__main__':
    unittest.main()



"""

"""
# !!!
import sql_raw.base_sql


def mutable_command(_connection, execute: str, params: tuple | dict | list, *args, **kwargs):
    """
    Декоратор для выполнения изменяемой SQL команды
    """
    with _connection.cursor() as cursor:
        cursor.execute(execute, params, multi=kwargs["multi"])
        _connection.commit()
    return cursor.statement


def read_command(_connection, execute: str, params: tuple | dict | list, *args, **kwargs):
    """
    Декоратор для выполнения чтения из БД
    """
    with _connection.cursor() as cursor:
        cursor.execute(execute, params, multi=kwargs["multi"])
        return cursor.fetchall()


try:
    from mysql.connector import connect, Error
    from mysql.connector.abstracts import MySQLConnectionAbstract

    sql_raw.base_sql.CONNECT = connect
    sql_raw.base_sql.ERROR = Error
    sql_raw.base_sql.READ_COMMAND_A = read_command
    sql_raw.base_sql.MUTABLE_COMMAND_A = mutable_command


    def Config(user: str, password: str, dbname: str | None = None,
               port: int = 3306,
               host: str = "localhost"):
        """
        Настройка SQL
        """
        sql_raw.base_sql.SETTINGS_DB = {"host": host,
                                        "port": port,
                                        "user": user,
                                        "dbname": dbname,
                                        "password": password}
except ModuleNotFoundError:
    raise ModuleNotFoundError("Установите `mysql-connector-python`")



from typing import Dict, Union, Any

from .base_file import BaseFile


class TxtFile(BaseFile):
    """
    Открывать текстового файла в текстовом и БИНАРНОМ виде на
    - чтение
    - запись
    - до записи стандартную
    """

    def __init__(self, name_file: str, *, mod: str = None, encoding: str = None, data: Any = None):

        tmp = name_file.split(".")
        if any((len(tmp) != 2, tmp[1] != "txt")):
            raise ValueError("Файл должен иметь расширение .txt")

        BaseFile.__init__(self, name_file)

        if mod:
            self.res = {
                "r": lambda: self.readFile(encoding=encoding),
                "w": lambda: self.writeFile(data=data),
                "rb": lambda: self.readBinaryFile(),
                "wb": lambda: self.writeBinaryFile(data=data),
                "a": lambda: self.appendFile(data=data),
                "ab": lambda: self.appendBinaryFile(data=data)

            }[mod]()

    def readFileToResDict(self, *args: str, separator: str = '\n') -> Dict[str, str]:
        """
        :param separator:
        :param args: Имя ключей словаря
        """
        resDict: Dict[str, str] = {}
        with open(self.name_file, "r") as f:
            for index, line in enumerate(f):
                resDict[args[index]] = line.replace(separator, "")
        return resDict

    def readFile(self, limit: int = 0, *, encoding: str = None) -> str:  # +
        with open(self.name_file, "r", encoding=encoding) as f:
            if limit:
                res: str = ""
                for line in f:
                    if limit:
                        res += line
                        limit -= 1
                    else:
                        break
                return res
            else:
                return f.read()

    def searchFile(self, name_find: str) -> bool:
        res = False
        with open(self.name_file, "r") as f:
            for line in f:
                if line.find(name_find) != -1:
                    res = True
                    break
        return res

    def readBinaryFile(self) -> bytes:  # +
        with open(self.name_file, "rb") as f:
            return f.read()

    def writeFile(self, data: str):  # +
        with open(self.name_file, "w") as f:
            f.write(data)

    def writeBinaryFile(self, data: Union[bytes, memoryview]):  # +
        with open(self.name_file, "wb") as f:
            f.write(data)

    def appendFile(self, data: str):  # +
        with open(self.name_file, "a") as f:
            f.write(data)

    def appendBinaryFile(self, data: bytes):  # +
        with open(self.name_file, "ab") as f:
            f.write(data)



from sql_raw.base_deco import Rsql, wsql
from sql_raw.postgres_sql import Config

Config(user="postgres", password="root", database="fast_api")


def test_read():
    print(Rsql("SELECT * FROM пользователь;"))


def test_write():
    wsql("INSERT INTO пользователь (id,f_name, l_name) VALUES (16,'t', 'd');")


def test_init():
    wsql("""    
    CREATE TABLE пользователь
    (
        id     serial PRIMARY KEY,
        f_name varchar(255) NOT NULL,
        l_name varchar(255) NOT NULL
    );
    INSERT INTO пользователь (id, f_name, l_name)
    VALUES (1, 'Carola', 'Yandle'),
           (2, 'Risa', 'Follet'),
           (3, 'Cele', 'Caslin'),
           (4, 'Osgood', 'Demead'),
           (5, 'Roldan', 'Malby'),
           (6, 'Reynard', 'Garlee'),
           (7, 'Erna', 'Vigurs'),
           (8, 'Stewart', 'Naismith'),
           (9, 'Poppy', 'Watling'),
           (10, 'Sybila', 'Teliga');
    CREATE TABLE фотографии
    (
        id         serial PRIMARY KEY,
        id_user    integer REFERENCES пользователь (id) ON DELETE CASCADE ON UPDATE CASCADE,
        path_image varchar(600)
    );
    INSERT INTO фотографии (id, id_user, path_image)
    VALUES (1, 1, 'http://dummyimage.com/212x100.png/dddddd/000000'),
           (2, 4, 'http://dummyimage.com/170x100.png/cc0000/ffffff'),
           (3, 5, 'http://dummyimage.com/147x100.png/dddddd/000000'),
           (4, 6, 'http://dummyimage.com/122x100.png/5fa2dd/ffffff'),
           (5, 2, 'http://dummyimage.com/222x100.png/dddddd/000000'),
           (6, 4, 'http://dummyimage.com/217x100.png/dddddd/000000'),
           (7, 1, 'http://dummyimage.com/192x100.png/cc0000/ffffff'),
           (8, 9, 'http://dummyimage.com/118x100.png/5fa2dd/ffffff'),
           (9, 6, 'http://dummyimage.com/215x100.png/cc0000/ffffff'),
           (10, 4, 'http://dummyimage.com/235x100.png/ff4444/ffffff');
    """)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mg_file',
 'mg_file.file',
 'mg_file.file.test',
 'mg_file.sql_raw',
 'mg_file.sql_raw.test',
 'mg_file.sqllite_orm',
 'mg_file.sqllite_orm.test']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=6.0,<7.0', 'loguru>=0.6.0,<0.7.0', 'prettytable>=3.0.0,<4.0.0']

setup_kwargs = {
    'name': 'mg-file',
    'version': '0.1.4',
    'description': 'Удобное пользование файлами',
    'long_description': '# Что это ?\n\nЭто встраемовая библиотека для работы с файлами формата\n\n- txt = [TxtFile](#txtfile)\n- csv = [CsvFile](#csvFile)\n- json = [JsonFile](#jsonfile)\n- pick = [PickleFile](#picklefile)\n- SqlLite = [SqlLiteQrm](#sqlLiteqrm)\n\n# Описание функционала библиотеки\n\n## TxtFile\n\nСначала создаем экземпляр класса `TxtFile` а потом работаем с его методами\n\n```python\ntxt_obj = TxtFile("test.txt")\n```\n\n---\n\n- `readFileToResDict(*args: str, separator: str = \'\\n\')-> Dict[str, str]` = Считывает файл и возвращает Dict с ключами\n  заданными в параметры `*args` разграничение происходит по параметру\n  `separator`\n\n> Пример\n\n```python\ntxt_obj.writeFile("my name\\nmy passwd\\nmy token")\nres = txt_obj.readFileToResDict("name", "passwd", "token")\nassert res == {\'name\': \'my name\', \'passwd\': \'my passwd\', \'token\': \'my token\'}\n```\n\n---\n\n- `readFile(limit: int = 0, *, encoding: str = None)-> str` = Обычное чтение `.txt` файла. Можно указать лимит по чтение\n  строчек `limit`. И кодировку чтения `encoding` значения такие же как и стандартной функции `open()`\n\n> Пример\n\n```python\ntest_text = "123123\\n3123133\\n12312d1d12313"\ntxt_obj.writeFile(test_text)\nassert txt_obj.readFile() == "123123\\n3123133\\n"\n```\n\n---\n\n- `searchFile(name_find:str) -> bool` = Поиск слова `name_find` в тексте\n\n> Пример\n\n```python\ntest_text = "Optional. If the number of \\n bytes returned exceed the hint number, \\n no more lines will be returned. Default value is  -1, which means all lines will be returned."\ntxt_obj.writeFile(test_text)\nassert txt_obj.searchFile("more") == True\n```\n\n---\n\n- `readBinaryFile()->bytes` = Чтение бинарного файла\n\n> Пример\n\n```python\ntest_str = \'123\'\ntxt_obj.writeBinaryFile(test_str.encode())\nassert test_str.encode() == txt_obj.readBinaryFile()\n```\n\n---\n\n- `writeFile(data:str)` = Запись в тактовом режиме\n\n---\n\n- `appendFile(data: str)` = Добавление в текстовом режиме\n\n---\n\n- `writeBinaryFile(data: Union[bytes, memoryview])` = Запись в бинарном режиме\n\n---\n\n- `appendBinaryFile(data: bytes)` = Добавление данных в бинарный файл\n\n---\n\n## CsvFile\n\nСначала создаем экземпляр класса `CsvFile` а потом работаем с его методами\n\n```python\ncsv_obj = CsvFile("test.csv")\n```\n\n- `readFile(encoding: str = "utf-8", newline: str = "", limit: int = None, miss_get_head=False) -> List[List[str]]` =\n  чтение cvs файла. `encodin newline` стандартной функции `open()`. `limit` сколько строк считать с начало.\n  `miss_get_head` вернуть данные без заголовков\n\n> Пример\n\n```python\ncsv_obj.writeFile(\n    [[1, 23, 41, 5],\n     [21, 233, 46, 35],\n     [13, 233, 26, 45],\n     [12, 213, 43, 56]], FlagDataConferToStr=True, header=("Данные", "Data", "Числа", "Num"))\n\nassert csv_obj.readFile() == [[\'Данные\', \'Data\', \'Числа\', \'Num\'],\n                              [\'1\', \'23\', \'41\', \'5\'],\n                              [\'21\', \'233\', \'46\', \'35\'],\n                              [\'13\', \'233\', \'26\', \'45\'],\n                              [\'12\', \'213\', \'43\', \'56\']]\n```\n\n---\n\n- `readFileAndFindDifferences(new_data_find: List[List], funIter) -> bool` = Чтение csv файла, с проверкой различий\n  входных данных с данными в файле, если такие различия найдены то выполняется переданная функция `funIter`.\n\n> Пример\n\n```python\ndata_file = [[\'1\', \'2\'], [\'3\', \'2\'], ["today", "Saturday"]]\nnew_data = [[\'1\', \'2\'], [\'3\', \'2\'], ["today", "Monday"]]\nDifferenceList = []\ncsv_obj.writeFile(data_file, header=("h1", "h2"))\ncsv_obj.readFileAndFindDifferences(new_data, DifferenceList.append)\nassert DifferenceList == [["today", "Saturday"]]\n```\n\n---\n\n- `readFileRevers(limit: int = None, encoding: str = "utf-8", newline: str = "") ->  List[List[str]]` = Чтение cvs файла\n  с конца в начало.\n  `encodin newline` стандартной функции `open()`. `limit` сколько строк считать с конца.\n\n> Пример\n\n```python\ncsv_obj.writeFile(\n    [[1, 23, 41, 5],\n     [21, 233, 46, 35],\n     [13, 233, 26, 45],\n     [12, 213, 43, 56]], FlagDataConferToStr=True, header=("Данные", "Data", "Числа", "Num"))\nassert csv_obj.readFileRevers() == [[\'12\', \'213\', \'43\', \'56\'],\n                                    [\'13\', \'233\', \'26\', \'45\'],\n                                    [\'21\', \'233\', \'46\', \'35\'],\n                                    [\'1\', \'23\', \'41\', \'5\'],\n                                    [\'Данные\', \'Data\', \'Числа\', \'Num\']]\n```\n\n---\n\n- `writeFile(data: Union[List[Union[str, int, float]], List[List[Union[str, int, float]]]], *, header: tuple = None, FlagDataConferToStr: bool = False, encoding: str = "utf-8", newline: str = "")`\n  = Запись данных в csv файл. `encodin newline` стандартной функции `open()`.\n  `FlagDataConferToStr` Проверять все входнее данные и конвертировать их в тип `str`. `header` = Задать заголовки\n  столбцам.\n\n---\n\n- `appendFile(data: Union[List[Union[str, int, float]], List[List[Union[str, int, float]]]], *, FlagDataConferToStr: bool = False, encoding: str = "utf-8", newline: str = ""\n  )` = Добавить данные в конец csv файла. Такие же параметры как у `writeFile.\n\n> Пример\n\n```python\ncsv_obj.writeFile(\n    [[1, 23, 41, 5],\n     [21, 233, 46, 35],\n     [13, 233, 26, 45],\n     [12, 213, 43, 56]], FlagDataConferToStr=True, header=("Данные", "Data", "Числа", "Num"))\n\ncsv_obj.appendFile([[\'2323\', \'23233\', \'23\']])\n\nassert csv_obj.readFile() == [[\'Данные\', \'Data\', \'Числа\', \'Num\'],\n                              [\'1\', \'23\', \'41\', \'5\'],\n                              [\'21\', \'233\', \'46\', \'35\'],\n                              [\'13\', \'233\', \'26\', \'45\'],\n                              [\'12\', \'213\', \'43\', \'56\'],\n                              [\'2323\', \'23233\', \'23\']]\n```\n\n---\n\n## JsonFile\n\nСначала создаем экземпляр класса `JsonFile` а потом работаем с его методами\n> Пример\n\n```python\njson_obj = JsonFile("test.json")\n```\n\n---\n\n- `readFile()` = Чтение данных из json файла\n\n---\n\n- `writeFile(data: Union[List, Dict], *, indent=4, ensure_ascii: bool = False)` = Запись данных в файл, входные\n  параметры такие же как у стандартной функции `open()`\n\n---\n\n- `appendFile(data: Union[List, Dict], *, ensure_ascii: bool = False)` = Добавить данные в файл\n\n> Пример\n\n```python\n# List\ntempers: List = [1, 2, 3, 4]\n\njson_obj.writeFile(tempers)\njson_obj.appendFile(tempers)\n\ntempers += tempers\nassert tempers == json_obj.readFile()\n\n# Dict\ntempers: Dict = {\'1\': 11, \'2\': 22, \'3\'::33}  # Все ключи должны быть типа str\n\njson_obj.writeFile(tempers)\njson_obj.appendFile(tempers)\n\ntempers.update(tempers)\nassert tempers == json_obj.readFile()\n```\n\n---\n\n## PickleFile\n\nСначала создаем экземпляр класса `PickleFile` а потом работаем с его методами\n> Пример\n\n```python\npick_obj = PickleFile("test.pkl")\n```\n\n---\n-`writeFile( data: Any, *, protocol: int = 3)` = Записать данные в pkl\n\n---\n-`readFile()` = Чтение данных phl\n> Пример\n\n```python\ntest_data = [\n    (1, 2, 3, 4),\n    [12, 23, 221],\n    ["1231", 12, (2, 22)],\n    {213123, 123213},\n    {\'s1\': \'213\'},\n]\nfor td in test_data:\n    pick_obj.writeFile(td)\n    assert pick_obj.readFile() == td\n    pick_obj.deleteFile()\n```\n\n---\n-`appendFile(data:  Union[Tuple, List, Dict, Set], *, protocol: int = 3)` = Добавить данные в pkl\n> Пример\n\n```python\ntest_data = [1, 2, 3, 4]\nnew_data = [98, 678, 88]\npick_obj.writeFile(test_data)\npick_obj.appendFile(new_data)\ntest_data += new_data\nassert pick_obj.readFile() == test_data\n```\n\n---\n\n## SqlLiteQrm\n\n```python\nsq = SqlLiteQrm(\'data_base.db\')\n```\n\n---\n\n- `.name_db` = Получить имя БД\n\n---\n\n- `.header_table[]` = Получить заголовок таблиц из БД\n\n---\n> Пример\n\n```python\ntest_header = {"id": PrimaryKey(int),\n               "name": toTypeSql(str),\n               "old": NotNullDefault(int, 5),\n               "salary": NotNull(float)}\nsq.CreateTable(\'name_table\', test_header)\nassert sq.header_table[\'name_table\'] == {\'id\': \'INTEGER PRIMARY KEY\',\n                                         \'name\': \'TEXT\',\n                                         \'old\': \'INTEGER NOT NULL DEFAULT 5\',\n                                         \'salary\': \'REAL NOT NULL\'}\n```\n\n---\n\n- `ListTables()` = Получить имена всех таблиц в БД\n\n---\n\n- `HeadTable(NameTable, width_table: int = 10)` = Выводи в консоль данные о таблицы из БД. `NameTable` имя таблицы,\n  `width_table` ширина столбцов.\n\n> Пример\n\n```python\nsq.CreateTable(\'name_table\', {"id": toTypeSql(int), "name": toTypeSql(str), "old": toTypeSql(int)})\nprint(sq.HeadTable(\'name_table\', 15))\n```\n\n---\n\n- `GetTable(NameTable: str, LIMIT: Tuple[int, int] = None, FlagPrint: int = 0)` = Получить данные из таблицы в\n  БД. `LIMIT(до, шаг)`, `FlagPrint` Указывает печатать ли результат в консоль\n\n> Пример\n\n```python\ntest_header = {"id": PrimaryKey(int),\n               "name": toTypeSql(str),\n               "old": NotNullDefault(int, 5),\n               "salary": NotNull(float)}\nsq.CreateTable(\'name_table\', test_header)\nassert sq.header_table[\'name_table\'],\n{\'id\': \'INTEGER PRIMARY KEY\', \'name\': \'TEXT\', \'old\': \'INTEGER NOT NULL DEFAULT 5\',\n \'salary\': \'REAL NOT NULL\'})\nsq.ExecuteTable(\'name_table\', {"id": 1, "name": "Anton", "old": 30, "salary": 3000.11})\nsq.ExecuteTable(\'name_table\', {"id": 2, "name": "Katy", "old": 22, "salary": 3200.23})\nassert sq.GetTable(\'name_table\') == [(1, \'Anton\', 30, 3000.11), (2, \'Katy\', 22, 3200.23)]\n```  \n\n--- \n\n- `GetColumn(NameTable: str, name_columns: str, LIMIT: Tuple[int, int] = None)` = Получить данные из столбца таблицы в\n  БД\n\n> Пример\n\n```python\nsq.CreateTable(self.name_table,\n               {"id": PrimaryKeyAutoincrement(int), "name": toTypeSql(str), "old": toTypeSql(int),\n                "sex": NotNullDefault(str, "_")})\nsq.ExecuteManyTableDict(self.name_table,\n                        [{"name": "Denis", "old": 21},\n                         {"name": "Katy", "old": 21, "sex": 1},\n                         {"name": "Svetha", "old": 24}]\n                        )\nassert self.sq.GetColumn(self.name_table, "name") == [\'Denis\', \'Katy\', \'Svetha\']\nassert self.sq.GetColumn(self.name_table, "old") == [21, 21, 24]\nassert self.sq.GetColumn(self.name_table, "old", LIMIT=(2, 0)) == [21, 21]\n```\n\n---\n\n- `DeleteTable(NameTable: Union[str, List[str]])` = Удалить одну таблицу или несколько таблиц\n\n---\n\n- `DeleteLineTable(NameTable: str, sqlWHERE: str = "")` = Удалить строки по условию `WHERE`\n\n> Пример\n\n```python\nsq.CreateTable(\'name_table\',\n               {"id": PrimaryKeyAutoincrement(int), "name": toTypeSql(str), "old": toTypeSql(int),\n                "sex": NotNullDefault(str, "_")})\nsq.ExecuteManyTableDict(\'name_table\', [{"name": "Denis", "old": 21},\n                                       {"name": "Katy", "old": 221, "sex": 1},\n                                       {"name": "Mush", "old": 321, "sex": 21},\n                                       {"name": "Patio", "old": 231, "sex": 21},\n                                       {"name": "Pvetha", "old": 24}])\nsq.DeleteLineTable(\'name_table\', "old > 25")\nassert sq.GetTable(\'name_table\') == [(1, \'Denis\', 21, \'_\'), (5, \'Pvetha\', 24, \'_\')]\n```\n\n---\n\n- `CreateTable(NameTable: str, columns: Union[str, Dict])` = Создать таблицу с заголовками столбцов `columns`.\n  Используйте вспомогательные функции из `sqlmodules` для удобного создание sql запросов\n\n> Пример\n\n```python\n# Должны содержать уникальные значения\nPrimaryKey = lambda TypeColumn: definition(TypeColumn) + " PRIMARY KEY"\n# Всегда должно быть заполнено\nNotNull = lambda TypeColumn: definition(TypeColumn) + " NOT NULL"\n# Все столбцы будут по умолчанию заполнены указанными значениями\nNotNullDefault = lambda TypeColumn, default: definition(TypeColumn) + f" NOT NULL DEFAULT {default}"\n# Значение по умолчанию\nDefault = lambda TypeColumn, default: definition(TypeColumn) + " DEFAULT {0}".format(default)\n# Авто заполнение строки. подходит для id\nPrimaryKeyAutoincrement = lambda TypeColumn: definition(TypeColumn) + " PRIMARY KEY AUTOINCREMENT"\n# Конвертация  типа данных python в SQLLite\ntoTypeSql = lambda TypeColumn: definition(TypeColumn)\n```\n\n- `ExecuteTable(NameTable: str, data: Union[str, int, float, List[Union[str, bytes, int, float]], Tuple, Dict[str, Union[str, bytes, int, float]]] = None, sqlRequest: str = "", *, CheckBLOB: bool = False)`\n  = Добавить запись в таблицу из БД\n  `sqlRequest` вы можете добавить данные из sql запрос.`CheckBLOB` если вы записываете бинарные данные в таблицу\n  используйте этот флаг.\n\n> Пример\n\n```python\nsq.CreateTable(\'name_table\',\n               {"id": toTypeSql(int),\n                "old": toTypeSql(int)\n                })\ntest_table: str = \'test_table\'\nsq.CreateTable(test_table,\n               {"old": toTypeSql(int)})\n\nsq.ExecuteManyTable(\'name_table\',\n                    [[11, 24],\n                     [22, 31],\n                     [2312, 312],\n                     [231, 68],\n                     [344, 187]])\n\nresSQL = Select(\'name_table\', "id").Where("id < 30").Request\n\nsq.ExecuteTable(test_table, sqlRequest=resSQL)\n\nassert sq.GetTable(test_table) == [(11,), (22,)]\n```\n\n> Пример `CheckBLOB`\n\n```python\ntest_header = {"str": toTypeSql(str), "int": toTypeSql(int), "float": toTypeSql(float),\n               "bytes": toTypeSql(bytes)}\nsq.CreateTable(\'name_table\', test_header)\ntest_data = ("text", 123, 122.32, b"1011")\nsq.ExecuteTable(\'name_table\', test_data, CheckBLOB=True)\nassert sq.GetTable(\'name_table\')[0] == test_data\n```            \n\n- `ExecuteManyTable(NameTable: str, data: List[Union[List[Union[str, bytes, Binary, int, float]], Tuple]], head_data: Union[List[str], Tuple] = None, *, CheckBLOB: bool = False)`\n  = Добавить несколько записей в таблицу в формате List. `head_data` указывать когда длинна входных данных меньше чем\n  количество столбцов в таблице\n\n> Пример\n\n```python\nsq.CreateTable(\'name_table\', {\n    \'car_id\': PrimaryKeyAutoincrement(int),\n    "model": toTypeSql(str),\n    "price": toTypeSql(int)\n})\n\ncar = [\n    ("Audi", 432),\n    ("Maer", 424),\n    ("Skoda", 122)\n]\nsq.ExecuteManyTable(\'name_table\', car, head_data=("model", "price"))\nassert sq.GetTable(\'name_table\') == [(1, \'Audi\', 432), (2, \'Maer\', 424), (3, \'Skoda\', 122)]\n```\n\n- `ExecuteManyTableDict(NameTable: str, data: List[Dict])` = Добавить несколько записей в таблицу в формате Dict\n\n> Пример\n\n```python\nsq.CreateTable(\'name_table\',\n               {"id": PrimaryKeyAutoincrement(int), "name": toTypeSql(str), "old": toTypeSql(int),\n                "sex": NotNullDefault(str, "_")})\nsq.ExecuteManyTableDict(\'name_table\',\n                        [{"name": "Denis", "old": 21},\n                         {"name": "Katy", "old": 21, "sex": 1},\n                         {"name": "Svetha", "old": 24}]\n                        )\nassert sq.Search(Select(\'name_table\', "name").Where("old == 21")) == [(\'Denis\',), (\'Katy\',)]\n```\n\n- `UpdateColumne(NameTable: str, name_column: Union[str, List[str]], new_data: Union[str, bytes, int, float, List[Union[str, bytes, int, float]]], sqlWHERE: str = "")`\n  = Обновить данные в столбцах таблицы по условию `sqlWHERE`\n\n> Пример\n\n```python\nsq.CreateTable(\'name_table\',\n               {"id": PrimaryKeyAutoincrement(int), "name": toTypeSql(str), "old": toTypeSql(int),\n                "sex": NotNullDefault(str, "_")})\nsq.ExecuteManyTableDict(\'name_table\', [{"name": "Denis", "old": 21},\n                                       {"name": "Katy", "old": 221, "sex": 1},\n                                       {"name": "Mush", "old": 321, "sex": 21},\n                                       {"name": "Patio", "old": 231, "sex": 21},\n                                       {"name": "Svetha", "old": 24}])\n\nsq.UpdateColumne(\'name_table\', \'old\', 99)\nassert sq.GetTable(\'name_table\') == [(1, \'Denis\', 99, \'_\'),\n                                     (2, \'Katy\', 99, \'1\'),\n                                     (3, \'Mush\', 99, \'21\'),\n                                     (4, \'Patio\', 99, \'21\'),\n                                     (5, \'Svetha\', 99, \'_\')]\n```\n\n- `Search` = Поиск данных в таблице\n- `SaveDbToFile` = Сохранить БД в отдельный\n- `ReadFileToDb`\n- `DeleteDb` \n\n\n\n\n\n\n\n\n\n',
    'author': 'Ваше Имя',
    'author_email': 'denis-kustov@rambler.ru',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/denisxab/mg_file',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from pprint import pformat

from loguru import logger

# !!!
import sql_raw.base_sql


def connect_db(fun):
    """
    Декоратор для создать подключение к БД
    """

    def wrapper(*arg, **kwargs):
        try:
            with sql_raw.base_sql.CONNECT(**sql_raw.base_sql.SETTINGS_DB) as connection:
                res = fun(connection, *arg, **kwargs)
            return res
        except sql_raw.base_sql.ERROR as e:
            logger.error(e)
            raise e

    return wrapper


def mutable_command(*args, **kwargs):
    """
    Декоратор для выполнения изменяемой SQL команды
    """

    def wrapper(fun):
        def decorated_function(_connection, *arg, **kwarg) -> str:
            return sql_raw.base_sql.MUTABLE_COMMAND_A(_connection, *fun(*arg, **kwargs), *args, *arg, **kwargs, **kwarg)

        return decorated_function

    return wrapper


def read_command(*args, **kwargs):
    """
    Декоратор для выполнения чтения из БД

    @param tdata: Тип возвращаемого объекта
        - "d" - dictfetchall
        - "n" - namedtuplefetchall
        - "o" - fetchone
        - "a" - fetchall
    """

    def wrapper(fun):
        def decorated_function(_connection, *arg, **kwarg) -> any:
            return sql_raw.base_sql.READ_COMMAND_A(_connection, *fun(*arg, **kwargs), *args, *arg,
                                                   **kwargs, **kwarg)

        return decorated_function

    return wrapper


def pprint_deco(fun):
    """
    Декоратор для красивого вывода результата функции в консоль
    """

    def wrapper(*arg, **kwargs):
        return pformat(fun(*arg, **kwargs))

    return wrapper


@connect_db
@read_command()
def rsql(execute: str, params: tuple | dict | list = ()) -> tuple[str, tuple | dict | list]:
    """
    Чтение из БД
    """
    return execute, params


@pprint_deco
@connect_db
@read_command()
def Rsql(execute: str, params: tuple | dict | list = ()) -> tuple[str, tuple | dict | list]:
    """
    Чтение из БД с красивым выводом в консоль
    """
    return execute, params


@connect_db
@mutable_command()
def wsql(execute: str, params: tuple | dict | list = ()) -> tuple[str, tuple | dict | list]:
    """
    Внесение изменений в БД
    """
    return execute, params
